//  $Id: alu_control.v,v 1.1.1.1 2007/04/20 23:07:44 vickery Exp $
/*  Hennessy and Patterson MIPS subset: ALU Control unit.
 *  Augmented to handle immediate op codes.
 *
 *  $Log: alu_control.v,v $
 *  Revision 1.1.1.1  2007/04/20 23:07:44  vickery
 *  Initial version
 *
 */
module alu_control(alu_func, aluOp, op_code, func_code);
input[1:0]  aluOp;
input[5:0]  op_code, func_code;
output[3:0] alu_func;

reg[3:0]    alu_func;

wire        op_addi, op_andi, op_beq, op_bne, op_j, op_jal, op_jr,
            op_lui, op_lw, op_ori, op_slti, op_sw;

wire        func_add, func_and, func_nor, func_or, func_slt, func_sub;

assign func_add = 6'h20 == func_code;
assign func_and = 6'h24 == func_code;
assign func_nor = 6'h27 == func_code;
assign func_or  = 6'h25 == func_code;
assign func_slt = 6'h2A == func_code;
assign func_sub = 6'h22 == func_code;

assign op_addi  = 6'h08 == op_code;
assign op_andi  = 6'h0C == op_code;
assign op_lui = 6'h0F == op_code;
assign op_ori = 6'h0D == op_code;
assign op_slti  = 6'h0A == op_code;

  always @(aluOp, func_add, func_and, func_nor, func_or, func_slt, func_sub,
           op_addi, op_andi, op_lui, op_ori, op_slti)
  begin
    case (aluOp)
      2'b00:  alu_func <= 4'b0010;
      2'b01:  alu_func <= 4'b0110;
      2'b10:  alu_func <= func_add ? 4'b0010 :
                          func_and ? 4'b0000 :
                          func_nor ? 4'b1100 :
                          func_or  ? 4'b0001 :
                          func_slt ? 4'b0111 : 
                          func_sub ? 4'b0110 : 4'b0000;
      2'b11:  alu_func <= op_addi  ? 4'b0010 :
                          op_andi  ? 4'b0000 :
                          op_lui   ? 4'b0001 :
                          op_ori   ? 4'b0001 :
                          op_slti  ? 4'b0111 : 4'b0000;
    endcase
  end

endmodule

