//  $Id: control_unit.v,v 1.2 2007/04/21 01:35:33 vickery Exp $
/*  MIPS Single-cycle control unit.
 *  C. Vickery
 *  April, 2007
 *
 *  $Log: control_unit.v,v $
 *  Revision 1.2  2007/04/21 01:35:33  vickery
 *  Fixed logic errors when testing op codes.
 *
 *  Revision 1.1.1.1  2007/04/20 23:07:45  vickery
 *  Initial version
 *
 */
module control_unit(  RegDst,
                      Jump,
                      Branch,
                      MemRead,
                      MemtoReg,
                      ALUop,
                      MemWrite,
                      ALUSrc,
                      RegWrite,
                      op_code);
input [5:0]   op_code;
output        RegDst, Jump, Branch, MemRead, MemtoReg, MemWrite,
              RegWrite, ALUSrc;
output [1:0]  ALUop;
reg           RegDst, Jump, Branch, MemRead, MemtoReg, MemWrite,
              RegWrite, ALUSrc;
reg [1:0]     ALUop;

wire          op_addi, op_andi, op_beq, op_bne, op_j,
              op_lui, op_lw, op_ori, op_slti, op_sw;

assign op_addi  = (6'h08 == op_code);
assign op_andi  = (6'h0C == op_code);
assign op_beq   = (6'h04 == op_code);
assign op_bne   = (6'h05 == op_code);
assign op_j     = (6'h02 == op_code);
assign op_lui   = (6'h0F == op_code);
assign op_lw    = (6'h23 == op_code);
assign op_ori   = (6'h0D == op_code);
assign op_slti  = (6'h0A == op_code);
assign op_sw    = (6'h2B == op_code);
assign r_format = (6'h00 == op_code);
assign i_format = op_lw | op_sw | op_addi | op_andi | op_lui | op_ori | op_slti;

  always
  begin
    RegDst    <= ~i_format;
    Jump      <= op_j;
    Branch    <= op_beq | op_bne;
    MemRead   <= op_lw;
    MemtoReg  <= op_lw;
    MemWrite  <= op_sw;
    RegWrite  <= ~(op_sw | op_beq | op_bne | op_j);
    ALUop     <=  (op_lw | op_sw)   ? 2'b00 :
                  (op_beq | op_bne) ? 2'b01 :
                  (r_format)      ? 2'b10 : 2'b11;
    ALUSrc    <= i_format;
  end
endmodule

